@@ -1472,13 +1472,39 @@ class BacktestingBroker(Broker):
 
             # Get the OHLCV data for the asset if we're using the PANDAS data source
             elif self.data_source.SOURCE == "PANDAS":
+                # Default to the current data source timestep, but force daily bars for daily-cadence strategies.
+                timestep = getattr(self.data_source, "_timestep", None) or self.data_source.get_timestep()
+                timeshift_value = -2
+                length_value = 2
+                if strategy is not None:
+                    use_daily = False
+                    cadence_seconds = None
+                    try:
+                        cadence_seconds = strategy._get_sleeptime_seconds()
+                    except Exception:
+                        cadence_seconds = None
+                    if cadence_seconds is not None and cadence_seconds >= 20 * 3600:
+                        use_daily = True
+                    base_asset = order.asset if not (isinstance(order.asset, tuple) and len(order.asset) > 0) else order.asset[0]
+                    if not use_daily:
+                        try:
+                            use_daily = strategy._should_use_daily_last_price(base_asset)
+                        except Exception:
+                            use_daily = False
+                    if not use_daily and getattr(self.data_source, "_timestep", None) == "day":
+                        use_daily = True
+                    if use_daily:
+                        timestep = "day"
+                        timeshift_value = -2
+                        length_value = 2
+
                 # This is a hack to get around the fact that we need to get the previous day's data to prevent lookahead bias.
                 ohlc = self.data_source.get_historical_prices(
                     asset=asset,
-                    length=2,
+                    length=length_value,
                     quote=order.quote,
-                    timeshift=-2,
-                    timestep=self.data_source._timestep,
+                    timeshift=timeshift_value,
+                    timestep=timestep,
                 )
                 # Check if we got any ohlc data
                 if ohlc is None or ohlc.empty:
@@ -1487,11 +1513,15 @@ class BacktestingBroker(Broker):
                         order_identifier = getattr(order, "identifier", None)
                         if order_identifier is None:
                             order_identifier = getattr(order, "id", "<unknown>")
+                        action = "holding" if timestep == "day" else "canceling"
                         strategy.log_message(
                             f"[DIAG] No pandas bars for {display_symbol} at {self.datetime}; "
-                            f"canceling {order.order_type} id={order_identifier}",
+                            f"{action} {order.order_type} id={order_identifier}",
                             color="yellow",
                         )
+                    if timestep == "day":
+                        # Leave the order pending so it can be re-priced on the next iteration when data is available.
+                        continue
                     self.cancel_order(order)
                     continue
 
@@ -1509,34 +1539,54 @@ class BacktestingBroker(Broker):
                         dt_col = 'datetime'  # fallback
 
                     # Filter for current time or future
-                    df = df_original.filter(pl.col(dt_col) >= self.datetime)
+                    df = df_original.filter(pl.col(dt_col) <= self.datetime)
 
                     # If the dataframe is empty, get the last row
                     if len(df) == 0:
+                        earliest_dt = df_original[dt_col].min()
+                        if earliest_dt is not None and self.datetime < earliest_dt:
+                            if strategy is not None:
+                                display_symbol = getattr(order.asset, "symbol", order.asset)
+                                strategy.log_message(
+                                    f"[DIAG] No historical coverage yet for {display_symbol} at {self.datetime}; "
+                                    f"waiting for first real bar {earliest_dt}",
+                                    color="yellow",
+                                )
+                            continue
                         df = df_original.tail(1)
 
                     # Get values
-                    dt = df[dt_col][0]
-                    open = df["open"][0]
-                    high = df["high"][0]
-                    low = df["low"][0]
-                    close = df["close"][0]
-                    volume = df["volume"][0]
+                    dt = df[dt_col][-1]
+                    open = df["open"][-1]
+                    high = df["high"][-1]
+                    low = df["low"][-1]
+                    close = df["close"][-1]
+                    volume = df["volume"][-1]
                 else:  # Pandas DataFrame
                     # Make sure that we are only getting the prices for the current time exactly or in the future
-                    df = df_original[df_original.index >= self.datetime]
+                    df = df_original[df_original.index <= self.datetime]
 
                     # If the dataframe is empty, then we should get the last row of the original dataframe
                     # because it is the best data we have
                     if len(df) == 0:
+                        earliest_dt = df_original.index.min()
+                        if earliest_dt is not None and self.datetime < earliest_dt:
+                            if strategy is not None:
+                                display_symbol = getattr(order.asset, "symbol", order.asset)
+                                strategy.log_message(
+                                    f"[DIAG] No historical coverage yet for {display_symbol} at {self.datetime}; "
+                                    f"waiting for first real bar {earliest_dt}",
+                                    color="yellow",
+                                )
+                            continue
                         df = df_original.iloc[-1:]
 
-                    dt = df.index[0]
-                    open = df["open"].iloc[0]
-                    high = df["high"].iloc[0]
-                    low = df["low"].iloc[0]
-                    close = df["close"].iloc[0]
-                    volume = df["volume"].iloc[0]
+                    dt = df.index[-1]
+                    open = df["open"].iloc[-1]
+                    high = df["high"].iloc[-1]
+                    low = df["low"].iloc[-1]
+                    close = df["close"].iloc[-1]
+                    volume = df["volume"].iloc[-1]
 
             #############################
             # Determine transaction price.
