@@ -1321,6 +1321,8 @@ class Broker(ABC):
                 if order is None:
                     continue
 
+                # this seems very broker specific by assuming "leg" is how they package multileg orders
+                # it should be removed if the broker specific _parse_broker_order() function handles multileg orders
                 # Check if it is a multileg order and Parse the legs
                 if isinstance(broker_order, dict) and "leg" in broker_order and isinstance(broker_order["leg"], list):
                     parsed_legs = []
@@ -1374,6 +1376,10 @@ class Broker(ABC):
         if hasattr(self, '_submit_orders'):
             return self._submit_orders(orders, **kwargs)
         else:
+            #if kwargs indicates multileg orders with a limit price, and broker does not support it, we should error out instead of submitting legs individually
+            if kwargs.get('is_multileg') and kwargs.get('order_type') == Order.OrderType.LIMIT:
+                raise NotImplementedError("Multileg limit orders are not supported by this broker")
+
             with ThreadPoolExecutor(
                 max_workers=self.max_workers,
                 thread_name_prefix=f"{self.name}_submitting_orders",
@@ -1412,9 +1418,38 @@ class Broker(ABC):
             for order in orders:
                 tasks.append(executor.submit(self.cancel_order, order))
 
-    def cancel_open_orders(self, strategy):
-        """cancel all open orders for a given strategy"""
-        orders = [o for o in self.get_tracked_orders(strategy) if o.is_active()]
+    def cancel_open_orders(self, strategy, orders: list[Order] | None = None):
+        """Cancel all open orders for a given strategy.
+
+        Parameters
+        ----------
+        strategy : str
+            Strategy name whose orders should be canceled.
+        orders : list[Order] | None
+            Optional pre-filtered orders list. When provided, the broker
+            skips re-fetching tracked orders and cancels the supplied
+            active orders immediately. If None, the broker will gather
+            the active orders itself.
+        """
+        if orders is None:
+            orders = [o for o in self.get_tracked_orders(strategy) if o.is_active()]
+        else:
+            orders = [o for o in orders if o.is_active()]
+        if not orders:
+            self.logger.debug("cancel_open_orders(strategy=%s) -> no active orders", strategy)
+            return []
+        order_ids = [
+            getattr(order, "identifier", None)
+            or getattr(order, "id", None)
+            or getattr(order, "order_id", None)
+            for order in orders
+        ]
+        self.logger.debug(
+            "cancel_open_orders(strategy=%s) -> active=%d ids=%s",
+            strategy,
+            len(orders),
+            order_ids,
+        )
         self.cancel_orders(orders)
 
     def wait_orders_clear(self, strategy, max_loop=5):
@@ -1487,10 +1522,31 @@ class Broker(ABC):
         """
         pos = self.get_tracked_position(strategy_name, asset)
         if pos and pos.quantity != 0:
+            self.logger.info(
+                "close_position(strategy=%s, asset=%s, fraction=%s) -> qty=%s",
+                strategy_name,
+                getattr(asset, "symbol", asset),
+                fraction,
+                pos.quantity,
+            )
             order = pos.get_selling_order(quote_asset=self.quote_assets and next(iter(self.quote_assets)))
             if fraction != 1.00:
                 order.quantity = order.quantity * fraction
+            order_id = getattr(order, "identifier", None) or getattr(order, "id", None) or getattr(order, "order_id", None)
+            self.logger.info(
+                "close_position(strategy=%s) submitting order %s qty=%s side=%s type=%s",
+                strategy_name,
+                order_id,
+                getattr(order, "quantity", None),
+                getattr(order, "side", None),
+                getattr(order, "order_type", None),
+            )
             return self.submit_order(order)
+        self.logger.info(
+            "close_position(strategy=%s, asset=%s) -> no tracked position or zero quantity",
+            strategy_name,
+            getattr(asset, "symbol", asset),
+        )
         return None
 
     # =========Subscribers/Strategies functions==============
@@ -1715,6 +1771,13 @@ class Broker(ABC):
             "asset.expiration": stored_order.asset.expiration if stored_order.asset is not None else None,
             "asset.asset_type": stored_order.asset.asset_type if stored_order.asset is not None else None,
         }
+        price_source = getattr(stored_order, "_price_source", None)
+        if price_source:
+            new_row["price_source"] = price_source
+            try:
+                delattr(stored_order, "_price_source")
+            except AttributeError:
+                pass
         # Create a DataFrame with the new row
         new_row_df = pd.DataFrame(new_row, index=[0])
 
