@@ -1,5 +1,7 @@
 from dataclasses import dataclass
 from datetime import date, datetime, timedelta
+from decimal import Decimal
+import math
 from typing import Any, Dict, List, Optional, Tuple, Union
 import warnings
 
@@ -23,6 +25,7 @@ class OptionMarketEvaluation:
     sell_price: Optional[float]
     used_last_price_fallback: bool
     max_spread_pct: Optional[float]
+    data_quality_flags: List[str]
 
 
 class OptionsHelper:
@@ -58,6 +61,54 @@ class OptionsHelper:
         self._liquidity_deprecation_warned = False
         self.strategy.log_message("OptionsHelper initialized.", color="blue")
 
+    @staticmethod
+    def _coerce_price(value: Any, field_name: str, flags: List[str], notes: List[str]) -> Optional[float]:
+        """Normalize quote values and record data quality issues."""
+        raw_value = value
+
+        if value is None:
+            flags.append(f"{field_name}_missing")
+            return None
+
+        try:
+            if isinstance(value, Decimal):
+                value = float(value)
+            else:
+                value = float(value)  # type: ignore[arg-type]
+        except (TypeError, ValueError):
+            flags.append(f"{field_name}_non_numeric")
+            notes.append(f"{field_name} value {raw_value!r} is non-numeric; dropping.")
+            return None
+
+        if math.isnan(value) or math.isinf(value):
+            flags.append(f"{field_name}_non_finite")
+            notes.append(f"{field_name} value {value!r} is not finite; dropping.")
+            return None
+
+        if value <= 0:
+            flags.append(f"{field_name}_non_positive")
+            notes.append(f"{field_name} value {value!r} is non-positive; dropping.")
+            return None
+
+        return value
+
+    @staticmethod
+    def has_actionable_price(evaluation: Optional["OptionMarketEvaluation"]) -> bool:
+        """Return True when the evaluation contains a usable buy price."""
+        if evaluation is None:
+            return False
+
+        price = evaluation.buy_price
+        if price is None:
+            return False
+
+        try:
+            price = float(price)
+        except (TypeError, ValueError):
+            return False
+
+        return math.isfinite(price) and price > 0 and not evaluation.spread_too_wide
+
     # ============================================================
     # Basic Utility Functions
     # ============================================================
@@ -66,8 +117,8 @@ class OptionsHelper:
                                  expiry: date, put_or_call: str = "call") -> Optional[Asset]:
         """
         Find a valid option with the given expiry and strike.
-        If no option is available (e.g. due to holidays/weekends), increment the expiry day-by-day
-        (up to 10 times) until a valid option is found.
+        First tries the requested strike, then searches nearby strikes from the option chain.
+        If no strikes work for this expiry, tries the next expiry date.
 
         Parameters
         ----------
@@ -86,55 +137,111 @@ class OptionsHelper:
             The valid option asset or None if not found.
         """
         self.strategy.log_message(f"Finding next valid option for {underlying_asset.symbol} at strike {rounded_underlying_price} and expiry {expiry}", color="blue")
-        loop_counter = 0
-        while True:
+
+        expiry_attempts = 0
+        while expiry_attempts < 10:
+            # Check if this expiry was previously marked as invalid
             for record in self.non_existing_expiry_dates:
                 if (record["underlying_asset_symbol"] == underlying_asset.symbol and
                     record["expiry"] == expiry):
                     self.strategy.log_message(f"Expiry {expiry} previously invalid for {underlying_asset.symbol}; trying next day.", color="yellow")
                     expiry += timedelta(days=1)
+                    expiry_attempts += 1
                     continue
 
-            option = Asset(
-                underlying_asset.symbol,
-                asset_type="option",
-                expiration=expiry,
-                strike=rounded_underlying_price,
-                right=put_or_call,
-                underlying_asset=underlying_asset,
-            )
-
-            # First check if quote data exists (preferred)
+            # Try to get option chain to find available strikes
             try:
-                quote = self.strategy.get_quote(option)
-                has_valid_quote = quote and (quote.bid is not None or quote.ask is not None)
-                if has_valid_quote:
-                    self.strategy.log_message(f"Found valid quote for option {option.symbol} at expiry {expiry}", color="blue")
-                    return option
+                chains = self.strategy.get_chains(underlying_asset)
+                self.strategy.log_message(f"Got chains for {underlying_asset.symbol}: {bool(chains)}", color="cyan")
+
+                if chains:
+                    # Get available strikes for this expiry
+                    available_strikes = chains.strikes(expiry, put_or_call.upper())
+                    self.strategy.log_message(f"Available strikes for {expiry}: {len(available_strikes) if available_strikes else 0} strikes", color="cyan")
+
+                    if not available_strikes:
+                        self.strategy.log_message(f"No strikes available for {put_or_call.upper()} on {expiry}; trying next expiry.", color="yellow")
+                        self.non_existing_expiry_dates.append({
+                            "underlying_asset_symbol": underlying_asset.symbol,
+                            "expiry": expiry,
+                        })
+                        expiry += timedelta(days=1)
+                        expiry_attempts += 1
+                        continue
+
+                    # Find the closest strike to our target
+                    closest_strike = min(available_strikes, key=lambda x: abs(x - rounded_underlying_price))
+                    self.strategy.log_message(f"Target strike {rounded_underlying_price} -> Closest available strike: {closest_strike}", color="green")
+
+                    # Create option with the closest available strike
+                    option = Asset(
+                        underlying_asset.symbol,
+                        asset_type="option",
+                        expiration=expiry,
+                        strike=closest_strike,
+                        right=put_or_call,
+                        underlying_asset=underlying_asset,
+                    )
+
+                    # Verify this option has price data
+                    try:
+                        quote = self.strategy.get_quote(option)
+                        has_valid_quote = quote and (quote.bid is not None or quote.ask is not None)
+                        if has_valid_quote:
+                            self.strategy.log_message(f"Found valid option: {option.symbol} {option.right} {option.strike} exp {option.expiration}", color="green")
+                            return option
+                    except Exception as e:
+                        self.strategy.log_message(f"Error getting quote for {option.symbol}: {e}", color="yellow")
+
+                    # Fallback to last price
+                    try:
+                        price = self.strategy.get_last_price(option)
+                        if price is not None:
+                            self.strategy.log_message(f"Found valid option (via last price): {option.symbol} {option.right} {option.strike} exp {option.expiration}", color="green")
+                            return option
+                    except Exception as e:
+                        pass
+
+                    # If closest strike didn't work, this expiry might be invalid
+                    self.strategy.log_message(f"Could not get price data for strike {closest_strike} on {expiry}; trying next expiry.", color="yellow")
+                    self.non_existing_expiry_dates.append({
+                        "underlying_asset_symbol": underlying_asset.symbol,
+                        "expiry": expiry,
+                    })
+                    expiry += timedelta(days=1)
+                    expiry_attempts += 1
+                    continue
+
             except Exception as e:
-                self.strategy.log_message(f"Error getting quote for {option.symbol}: {e}", color="yellow")
+                self.strategy.log_message(f"Error getting chains for {underlying_asset.symbol}: {e}; falling back to direct strike attempt.", color="yellow")
+                # Fallback: Try the exact strike requested (old behavior)
+                option = Asset(
+                    underlying_asset.symbol,
+                    asset_type="option",
+                    expiration=expiry,
+                    strike=rounded_underlying_price,
+                    right=put_or_call,
+                    underlying_asset=underlying_asset,
+                )
 
-            # Fallback to checking last price if no quote
-            try:
-                price = self.strategy.get_last_price(option)
-                self.strategy.log_message(f"Price for option {option.symbol} at expiry {expiry} is {price}", color="blue")
-            except Exception as e:
-                self.strategy.log_message(f"Error getting price for {option.symbol}: {e}", color="red")
-                price = None
+                try:
+                    price = self.strategy.get_last_price(option)
+                    if price is not None:
+                        return option
+                except Exception:
+                    pass
 
-            if price is not None:
-                return option
-
-            self.strategy.log_message(f"No price found for option {option.symbol} at expiry {expiry}.", color="yellow")
+            # No valid option found for this expiry, try next day
+            self.strategy.log_message(f"No valid option found for expiry {expiry}; trying next expiry.", color="yellow")
             self.non_existing_expiry_dates.append({
                 "underlying_asset_symbol": underlying_asset.symbol,
                 "expiry": expiry,
             })
             expiry += timedelta(days=1)
-            loop_counter += 1
-            if loop_counter >= 10:
-                self.strategy.log_message("Exceeded maximum attempts to find a valid option.", color="red")
-                return None
+            expiry_attempts += 1
+
+        self.strategy.log_message("Exceeded maximum attempts to find a valid option.", color="red")
+        return None
 
     def get_strike_deltas(self, underlying_asset: Asset, expiry: date, strikes: List[float],
                           right: str, stop_greater_than: Optional[float] = None,
@@ -411,6 +518,9 @@ class OptionsHelper:
         buy_price: Optional[float] = None
         sell_price: Optional[float] = None
 
+        data_quality_flags: List[str] = []
+        sanitization_notes: List[str] = []
+
         # Attempt to get quotes first
         quote = None
         try:
@@ -422,24 +532,20 @@ class OptionsHelper:
             )
 
         if quote and quote.bid is not None and quote.ask is not None:
-            try:
-                bid = float(quote.bid)
-                ask = float(quote.ask)
-            except (TypeError, ValueError):
-                bid = quote.bid
-                ask = quote.ask
+            bid = self._coerce_price(quote.bid, "bid", data_quality_flags, sanitization_notes)
+            ask = self._coerce_price(quote.ask, "ask", data_quality_flags, sanitization_notes)
             has_bid_ask = bid is not None and ask is not None
 
         if has_bid_ask and bid is not None and ask is not None:
             buy_price = ask
             sell_price = bid
-            mid = (ask + bid) / 2 if (ask is not None and bid is not None) else None
-            if mid and mid > 0:
+            mid = (ask + bid) / 2
+            if not math.isfinite(mid) or mid <= 0:
+                spread_pct = None
+            else:
                 spread_pct = (ask - bid) / mid
                 if max_spread_pct is not None:
                     spread_too_wide = spread_pct > max_spread_pct
-            else:
-                spread_pct = None
         else:
             missing_bid_ask = True
 
@@ -454,6 +560,10 @@ class OptionsHelper:
 
         if last_price is None:
             missing_last_price = True
+        else:
+            last_price = self._coerce_price(last_price, "last_price", data_quality_flags, sanitization_notes)
+            if last_price is None:
+                missing_last_price = True
 
         if not has_bid_ask and allow_fallback and last_price is not None:
             buy_price = last_price
@@ -463,6 +573,14 @@ class OptionsHelper:
                 f"Using last-price fallback for {option_asset} due to missing bid/ask quotes.",
                 color="yellow",
             )
+        elif not has_bid_ask and allow_fallback and last_price is None:
+            data_quality_flags.append("last_price_unusable")
+
+        if buy_price is not None and (not math.isfinite(buy_price) or buy_price <= 0):
+            sanitization_notes.append(f"buy_price {buy_price!r} is not actionable; clearing.")
+            data_quality_flags.append("buy_price_non_finite")
+            buy_price = None
+            sell_price = None
 
         # Compose log message
         spread_str = f"{spread_pct:.2%}" if spread_pct is not None else "None"
@@ -470,6 +588,12 @@ class OptionsHelper:
         log_color = "red" if spread_too_wide else (
             "yellow" if (missing_bid_ask or missing_last_price or used_last_price_fallback) else "blue"
         )
+        if sanitization_notes:
+            note_summary = "; ".join(sanitization_notes)
+            self.strategy.log_message(
+                f"Option data sanitization for {option_asset}: {note_summary}",
+                color="yellow",
+            )
         self.strategy.log_message(
             (
                 f"Option market evaluation for {option_asset}: "
@@ -477,7 +601,8 @@ class OptionsHelper:
                 f"max_spread={max_spread_str}, missing_bid_ask={missing_bid_ask}, "
                 f"missing_last_price={missing_last_price}, spread_too_wide={spread_too_wide}, "
                 f"used_last_price_fallback={used_last_price_fallback}, "
-                f"buy_price={buy_price}, sell_price={sell_price}"
+                f"buy_price={buy_price}, sell_price={sell_price}, "
+                f"data_quality_flags={data_quality_flags}"
             ),
             color=log_color,
         )
@@ -495,6 +620,7 @@ class OptionsHelper:
             sell_price=sell_price,
             used_last_price_fallback=used_last_price_fallback,
             max_spread_pct=max_spread_pct,
+            data_quality_flags=data_quality_flags,
         )
 
     def check_option_liquidity(self, option_asset: Asset, max_spread_pct: float) -> bool:
@@ -555,6 +681,34 @@ class OptionsHelper:
         self.strategy.log_message(f"Order details: {details}", color="blue")
         return details
 
+    def _chain_hint(self, min_expiration_date):
+        """Temporarily set chain constraints on the underlying data source."""
+        broker = getattr(self.strategy, "broker", None)
+        data_source = getattr(broker, "data_source", None) if broker else None
+
+        class _ChainHintContext:
+            def __init__(self, ds, min_dt):
+                self.ds = ds
+                self.min_dt = min_dt
+                self.prev = None
+
+            def __enter__(self):
+                if not self.ds:
+                    return
+                self.prev = getattr(self.ds, "_chain_constraints", None)
+                self.ds._chain_constraints = {"min_expiration_date": self.min_dt}
+
+            def __exit__(self, exc_type, exc_val, exc_tb):
+                if not self.ds:
+                    return
+                if self.prev is None:
+                    if hasattr(self.ds, "_chain_constraints"):
+                        delattr(self.ds, "_chain_constraints")
+                else:
+                    self.ds._chain_constraints = self.prev
+
+        return _ChainHintContext(data_source, min_expiration_date)
+
     def get_expiration_on_or_after_date(self, dt: Union[date, datetime], chains: Union[Dict[str, Any], Chains], call_or_put: str, underlying_asset: Optional[Asset] = None) -> Optional[date]:
         """
         Get the expiration date that is on or after a given date, validating that the option has tradeable data.
@@ -602,6 +756,18 @@ class OptionsHelper:
             )
             return None
 
+        def _try_resolve_expiration(chain_map: Dict[str, Any]) -> List[Tuple[str, date]]:
+            expiration_dates: List[Tuple[str, date]] = []
+            for expiry_str in chain_map.keys():
+                try:
+                    from lumibot.entities.chains import _normalise_expiry
+                    expiry_date = _normalise_expiry(expiry_str)
+                    expiration_dates.append((expiry_str, expiry_date))
+                except Exception:
+                    continue
+            expiration_dates.sort(key=lambda x: x[1])
+            return expiration_dates
+
         # Get underlying symbol for validation
         underlying_symbol = None
         if underlying_asset:
@@ -612,18 +778,38 @@ class OptionsHelper:
             underlying_symbol = chains_map['UnderlyingSymbol']
 
         # Convert string expiries to dates for comparison
-        expiration_dates: List[Tuple[str, date]] = []
-        for expiry_str in specific_chain.keys():
-            try:
-                from lumibot.entities.chains import _normalise_expiry
-                expiry_date = _normalise_expiry(expiry_str)
-                expiration_dates.append((expiry_str, expiry_date))
-            except:
-                continue
-
-        expiration_dates.sort(key=lambda x: x[1])
+        expiration_dates: List[Tuple[str, date]] = _try_resolve_expiration(specific_chain)
         future_candidates = [(s, d) for s, d in expiration_dates if d >= dt]
 
+        # If we couldn't find any expirations beyond the requested date, attempt a deeper fetch
+        if not future_candidates and underlying_asset is not None:
+            self.strategy.log_message(
+                f"No expirations >= {dt} found in cached chains; requesting extended range...",
+                color="yellow",
+            )
+            with self._chain_hint(dt):
+                refreshed_chains = self.strategy.get_chains(underlying_asset)
+            if refreshed_chains:
+                chains_map = refreshed_chains if isinstance(refreshed_chains, dict) else {}
+                options_map = chains_map.get("Chains") if isinstance(chains_map.get("Chains"), dict) else None
+                if options_map:
+                    specific_chain = options_map.get(call_or_put_caps) if isinstance(options_map.get(call_or_put_caps), dict) else None
+                    if specific_chain:
+                        expiration_dates = _try_resolve_expiration(specific_chain)
+                        future_candidates = [(s, d) for s, d in expiration_dates if d >= dt]
+                        chain_refetched = True
+            if future_candidates:
+                self.strategy.log_message(
+                    f"Extended chain request delivered {len(future_candidates)} expirations >= {dt}.",
+                    color="blue",
+                )
+            else:
+                self.strategy.log_message(
+                    f"Extended chain request still lacks expirations on/after {dt}; giving up.",
+                    color="red",
+                )
+                return None
+
         # Check each candidate expiry to find one with valid data
         for exp_str, exp_date in future_candidates:
             strikes = specific_chain.get(exp_str)
@@ -665,18 +851,9 @@ class OptionsHelper:
                     self.strategy.log_message(f"Cannot validate data without underlying symbol, returning {exp_date}", color="yellow")
                     return exp_date
 
-        # No future expirations with valid data; log and check last available
-        if expiration_dates:
-            # Check the last available expiry for data
-            for exp_str, exp_date in reversed(expiration_dates):
-                strikes = specific_chain.get(exp_str)
-                if strikes and len(strikes) > 0:
-                    self.strategy.log_message(
-                        f"No valid expirations on or after {dt}; using latest available {exp_date} for {call_or_put_caps}.",
-                        color="yellow",
-                    )
-                    return exp_date
-
+        # No future expirations with tradeable data; let the caller skip entries gracefully.
+        msg = f"No valid expirations on or after {dt} with tradeable data for {call_or_put_caps}; skipping."
+        self.strategy.log_message(msg, color="yellow")
         return None
 
     # ============================================================
