@@ -5,6 +5,7 @@ from .bar import Bar
 from .bars import Bars as _BarsBase
 from .chains import Chains
 from .data import Data as _DataBase
+from .data_polars import DataPolars
 from .dataline import Dataline
 from .order import Order
 from .position import Position
@@ -14,3 +15,17 @@ from .trading_fee import TradingFee
 # Use base implementations directly
 Bars = _BarsBase
 Data = _DataBase
+__all__ = [
+    "Asset",
+    "AssetsMapping",
+    "Bar",
+    "Bars",
+    "Chains",
+    "Data",
+    "DataPolars",
+    "Dataline",
+    "Order",
+    "Position",
+    "Quote",
+    "TradingFee",
+]
