@@ -1,6 +1,7 @@
 from decimal import Decimal
 
 import lumibot.entities as entities
+from lumibot.entities.asset import StrEnum #todo: this should be centralized, and not repeated in Asset and Position
 
 
 class Position:
@@ -26,8 +27,36 @@ class Position:
         The assets that are free in the portfolio. (Crypto: only)
     avg_fill_price : float
         The average fill price of the position.
+    current_price : float
+        The current price of the asset.
+    market_value : float
+        The market value of the position.
+    pnl : float
+        The profit and loss of the position.
+    pnl_percent : float
+        The profit and loss of the position as a percentage of the average fill price.
+    asset_type : str
+        The type of the asset.
+    exchange : str
+        The exchange that the position is on.
+    currency : str
+        The currency that the position is denominated in.
+    multiplier : float
+        The multiplier of the asset.
+    expiration : datetime.date
+        The expiration of the asset. (Options and futures: only). Probably better to use on position.asset
+    strike : float
+        The strike price of the asset. (Options: only). Probably better to use on position.asset
+    option_type : str
+        The type of the option. (Options: only). Probably better to use on position.asset
+    side : PositionSide
+        The side of the position (LONG or SHORT)
     """
 
+    class PositionSide(StrEnum):
+        LONG = "LONG"
+        SHORT = "SHORT"
+
     def __init__(
             self,
             strategy,
@@ -38,6 +67,11 @@ class Position:
             available=0,
             avg_fill_price=None
         ):
+        """Creates a position.
+
+        NOTE: There are some properties that can be assigned to a position entity outside of the constructor (pnl, current_price, etc)
+
+        """
         self.strategy = strategy
         self.asset = asset
         self.symbol = self.asset.symbol
@@ -223,13 +257,13 @@ class Position:
             result['currency'] = self.currency
         if hasattr(self, 'multiplier'):
             result['multiplier'] = self.multiplier
-        if hasattr(self, 'expiration'):
+        if hasattr(self, 'expiration'):  #should probably use position.asset instead
             result['expiration'] = str(self.expiration) if self.expiration else None
-        if hasattr(self, 'strike'):
+        if hasattr(self, 'strike'): #should probably use position.asset instead
             result['strike'] = float(self.strike) if self.strike else None
-        if hasattr(self, 'option_type'):
+        if hasattr(self, 'option_type'): #should probably use position.asset instead
             result['option_type'] = self.option_type
-        if hasattr(self, 'underlying_symbol'):
+        if hasattr(self, 'underlying_symbol'): #should probably use position.asset instead
             result['underlying_symbol'] = self.underlying_symbol
 
         # Handle orders carefully - ensure to_dict() is called properly
